﻿
///  <license>
///
///  Copyright 2021 ChapmanWorld LLC ( https://chapmanworld.com )
///
///  Redistribution and use in source and binary forms, with or without modification,
///  are permitted provided that the following conditions are met:
///
///  1. Redistributions of source code must retain the above copyright notice,
///     this list of conditions and the following disclaimer.
///
///  2. Redistributions in binary form must reproduce the above copyright notice,
///     this list of conditions and the following disclaimer in the documentation and/or
///     other materials provided with the distribution.
///
///  3. Neither the name of the copyright holder nor the names of its contributors may be
///     used to endorse or promote products derived from this software without specific prior
///     written permission.
///
///  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR
///  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
///  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
///  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
///  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
///  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
///  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
///  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
///
///  </license>
unit cwPosix;

(*
   This is by no means a complete translation of the posix headers, far from it.
   Instead, this is an import for those headers required by the cwRuntime only.
*)

interface
{$ifndef MSWINDOWS}
{$ifdef fpc}{$mode objfpc}{$endif}

type
  TSockLen = uint32;
  TIntAddr = int32;

  TSockAddr = record
    sa_family: uint16;
    sa_data: array [0..13] of uint8;
    padding: array[0..15] of uint8;
  end;
  pTSockAddr = ^TSockAddr;

  TInAddr = record
    b1: uint8;
    b2: uint8;
    b3: uint8;
    b4: uint8;
  end;

  TSockAddrIn = record
    sin_family: int16;
      sin_port: uint16;
      sin_addr: TInAddr;
      sin_zero: array [0..7] of uint8; // ansichars
  end;

  TInAddr6 = record
    case integer of
      1: ( addr8: array [0..15] of uint8);
      2: (addr16: array [0..7]  of uint16);
      3: (addr32: array [0..3]  of int32);
  end;

  TSockAddrIn6 = record
      sin_family: int16;
        sin_port: uint16;
    sin_flowinfo: uint16;
        sin_addr: TInAddr6;
    sin_scope_id: uint32;
  end;

  TSocketHandle = int32;

  pthread_fastlock = record
    status   : nativeint;
    spinlock : int32;
  end;

  pthread_cond_t = record
    c_lock    : pthread_fastlock;
    c_waiting : pointer;
    padding   : array[0..pred(48)-sizeof(pthread_fastlock)-sizeof(pointer)-sizeof(int64)] of byte;
    align     : int64;
  end;
  ppthread_cond_t = ^pthread_cond_t;

  ppthread_list_t = ^pthread_list_t;
  pthread_list_t = record
    prev: ppthread_list_t;
    next: ppthread_list_t;
  end;

  pthread_mutex_s = record
    lock    : int32;
    count   : int32;
    owner   : int32;
    nusers  : int32;
    kind    : int32;
    spins   : int16;
    elision : int16;
    list    : pthread_list_t;
  end;

  pthread_mutex_t = record
    case int32 of
      0: (data: pthread_mutex_s);
      1: (size: array [0..40-1] of uint8);
      2: (align: int32);
  end;
  ppthread_mutex_t = ^pthread_mutex_t;

  pthread_mutexattr_t = record
    case int32 of
    0: ( size: array [0..4-1] of uint8 );
    1: ( align: int32 );
  end;
  ppthread_mutexattr_t = ^pthread_mutexattr_t;

  pthread_condattr_t = record
    case int32 of
      0: (size: array [0..4-1] of uint8);
      1: (align: int32);
  end;
  ppthread_condattr_t = ^pthread_condattr_t;

  time_t = nativeint;

  timespec = record
    tv_sec: time_t;
    tv_nsec: int64;
  end;

  ptimespec = ^timespec;

  tm = record
    tm_sec   : int32;      // seconds
    tm_min   : int32;      // minutes
    tm_hour  : int32;      // hours
    tm_mday  : int32;      // day of the month
    tm_mon   : int32;      // month
    tm_year  : int32;      // yea
    tm_wday  : int32;      // day of the week
    tm_yday  : int32;      // day in the year
    tm_isdst : int32;      // daylight saving time
  end;
  ptm = ^tm;

  clockid_t = int32;
  pclockid = ^clockid_t;

  pthread_t = uint64;

  pthread_start_routine = function( Arg: pointer ): pointer; cdecl;

  sched_param = record
    sched_priority : int32;
  end;

  pthread_attr_t = record
    detachstate   : int32;
    schedpolicy   : int32;
    schedparam    : sched_param;
    inheritsched  : int32;
    scope         : int32;
    guardsize     : nativeint;
    stackaddrSet  : int32;
    stackaddr     : pointer;
    stacksize     : nativeint;
  end;

const //- IP Protocols
  IPPROTO_UNSPEC   = 0;
  IPPROTO_IP       = IPPROTO_UNSPEC;
  IPPROTO_HOPOPTS  = IPPROTO_UNSPEC;
  IPPROTO_ICMP     = 1;
  IPPROTO_IGMP     = 2;
  BTHPROTO_RFCOMM  = 3;
  IPPROTO_IPIP     = 4;
  IPPROTO_TCP      = 6;
  IPPROTO_EGP      = 8;
  IPPROTO_PUP      = 12;
  IPPROTO_UDP      = 17;
  IPPROTO_IDP      = 22;
  IPPROTO_TP       = 29;
  IPPROTO_IPV6     = 41;
  IPPROTO_ROUTING  = 43;
  IPPROTO_FRAGMENT = 44;
  IPPROTO_RSVP     = 46;
  IPPROTO_GRE      = 47;
  IPPROTO_ESP      = 50;
  IPPROTO_AH       = 51;
  IPPROTO_ICMPV6   = 58;
  IPPROTO_NONE     = 59;
  IPPROTO_DSTOPTS  = 60;
  IPPROTO_MTP      = 92;
  IPPROTO_ENCAP    = 98;
  IPPROTO_PIM      = 103;
  IPPROTO_COMP     = 108;
  IPPROTO_RM       = 113;
  IPPROTO_SCTP     = 132;
  IPPROTO_RAW      = 255;
  IPPROTO_MAX      = 255;

const //- Socket types
  {$ifndef SOCKTYPES}
  {$ifdef LINUX}
  {$ifdef CPUMIPS}
  SOCK_DGRAM      = 1;
  SOCK_STREAM     = 2;
  SOCK_RAW        = 3;
  SOCK_RDM        = 4;
  SOCK_SEQPACKET  = 5;
  {$define SOCKTYPES}
  {$endif}
  {$endif}
  {$endif}

  {$ifndef SOCKTYPES}
  SOCK_STREAM     = 1;
  SOCK_DGRAM      = 2;
  SOCK_RAW        = 3;
  SOCK_RDM        = 4;
  SOCK_SEQPACKET  = 5;
  {$define SOCKTYPES}
  {$endif}

const //- Address Family
  AF_UNSPEC       = 0;  // From MSDN - The address family is unspecified.
  AF_UNIX         = 1;
  AF_LOCAL        = 1;
  AF_INET         = 2;
  AF_AX25         = 3;
  AF_NETROM       = 6;
  AF_BRIDGE       = 7;
  AF_ATMPVC       = 8;
  AF_X25          = 9;
  AF_ROSE         = 11;
  AF_DECnet       = 12;
  AF_NETBEUI      = 13;
  AF_SECURITY     = 14;
  AF_KEY          = 15;
  AF_NETLINK      = 16;
  AF_ROUTE        = 16;
  AF_NETBIOS      = 17;
  AF_PACKET       = 17;
  AF_ASH          = 18;
  AF_ECONET       = 19;
  AF_ATMSVC       = 20;
  AF_SNA          = 22;
  AF_PPPOX	      = 24;
  AF_WANPIPE      = 25;
  AF_LLC          = 26;
  AF_TIPC         = 30;
  AF_BLUETOOTH    = 31;
  AF_BTH          = 32;
  AF_IPX          = 4;
  AF_APPLETALK    = 5;
  AF_INET6        = 10;
  AF_IRDA         = 23;

const // Common sockets constants
  INVALID_SOCKET  = -1;
  SOCKET_ERROR    = -1;
  SOMAXCONN       = $7fffffff;
  FIONBIO         = $5421;
  FIONREAD        = $541B;
  SD_RECEIVE      = 0;
  SD_SEND         = 1;
  SD_BOTH         = 2;

const // Error constants
  EPERM           = 1;
  ENOENT          = 2;
  ESRCH           = 3;
  EINTR           = 4;
  EIO             = 5;
  ENXIO           = 6;
  E2BIG           = 7;
  ENOEXEC         = 8;
  EBADF           = 9;
  ECHILD          = 10;
  EAGAIN          = 11;
  ENOMEM          = 12;
  EACCESS         = 13;
  EFAULT          = 14;
  ENOTBLK         = 15;
  EBUSY           = 16;
  EEXIST          = 17;
  EXDEV           = 18;
  ENODEV          = 19;
  ENOTDIR         = 20;
  EISDIR          = 21;
  EINVAL          = 22;
  ENFILE          = 23;
  EMFILE          = 24;
  ENOTTY          = 25;
  ETXTBSY         = 26;
  EFBIG           = 27;
  ENOSPC          = 28;
  ESPIPE          = 29;
  EROFS           = 30;
  EMLINK          = 31;
  EPIPE           = 32;
  EDOM            = 33;
  ERANGE          = 34;
  EDEADLK         = 35;
  ENAMETOOLONG    = 36;
  ENOLCK          = 37;
  ENOSYS          = 38;
  ENOTEMPTY       = 39;
  ELOOP           = 40;
  EWOULDBLOCK     = EAGAIN;
  ENOMSG          = 42;
  EIDRM           = 43;
  ECHRNG          = 44;
  EL2NSYNC        = 45;
  EL3HLT          = 46;
  EL3RST          = 47;
  ELNRNG          = 48;
  EUNATCH         = 49;
  ENOCSI          = 50;
  EL2HLT          = 51;
  EBADE           = 52;
  EBADR           = 53;
  EXFULL          = 54;
  ENOANO          = 55;
  EBADRQC         = 56;
  EBADSLT         = 57;
  EDEADLOCK       = EDEADLK;
  EBFONT          = 59;
  ENOSTR          = 60;
  ENODATA         = 61;
  ETIME           = 62;
  ENOSR           = 63;
  ENONET          = 64;
  ENOPKG          = 65;
  EREMOTE         = 66;
  ENOLINK         = 67;
  EADV            = 68;
  ESRMNT          = 69;
  ECOMM           = 70;
  EPROTO          = 71;
  EMULTIHOP       = 72;
  EDOTDOT         = 73;
  EBADMSG         = 74;
  EOVERFLOW       = 75;
  ENOTUNIQ        = 76;
  EBADFD          = 77;
  EREMCHG         = 78;
  ELIBACC         = 79;
  ELIBBAD         = 80;
  ELIBSCN         = 81;
  ELIBMAX         = 82;
  ELIBEXEC        = 83;
  EILSEQ          = 84;
  ERESTART        = 85;
  ESTRPIPE        = 86;
  EUSERS          = 87;
  ENOTSOCK        = 88;
  EDESTADDRREQ    = 89;
  EMSGSIZE        = 90;
  EPROTOTYPE      = 91;
  ENOPROTOOPT     = 92;
  EPROTONOSUPPORT = 93;
  ESOCKTNOSUPPORT = 94;
  EOPNOTSUPP      = 95;
  EPFNOSUPPORT    = 96;
  EAFNOSUPPORT    = 97;
  EADDRINUSE      = 98;
  EADDRNOTAVAIL   = 99;
  ENETDOWN        = 100;
  ENETUNREACH     = 101;
  ENETRESET       = 102;
  ECONNABORTED    = 103;
  ECONNRESET      = 104;
  ENOBUFS         = 105;
  EISCONN         = 106;
  ENOTCONN        = 107;
  ESHUTDOWN       = 108;
  ETOOMANYREFS    = 109;
  ETIMEDOUT       = 110;
  ECONNREFUSED    = 111;
  EHOSTDOWN       = 112;
  EHOSTUNREACH    = 113;
  EALREADY        = 114;
  EINPROGRESS     = 115;
  ESTALE          = 116;
  EUCLEAN         = 117;
  ENOTNAM         = 118;
  ENAVAIL         = 119;
  EISNAM          = 120;
  EREMOTEIO       = 121;
  EDQUOT          = 122;
  ENOMEDIUM       = 123;
  EMEDIUMTYPE     = 124;

const
  ESysETIMEDOUT   = 60;

const
  CLOCKS_PER_SEC            = 1000000;
  CLOCK_REALTIME            = 0;
  CLOCK_MONOTONIC           = 1;
  CLOCK_PROCESS_CPUTIME_ID  = 2;
  CLOCK_THREAD_CPUTIME_ID   = 3;
  CLOCK_MONOTONIC_RAW       = 4;
  TIMER_ABSTIME             = $01;

const
    cSuccess = 0;
    cRTLD_LAZY = 1;

const
  {$ifdef MACOS}
  cLibDLName   = 'libdl.dylib';
  cLibDLPrefix = '_';
  {$else}
  cLibDLName   = 'libdl.so';
  cLibDLPrefix = '';
  {$endif}


function dlopen      ( filename: pansichar; flags: int32        ): pointer; cdecl; external cLibDLName name cLibDLPrefix+'dlopen';
function dlsym       ( handle: pointer;   symbolname: pansichar ): pointer; cdecl; external cLibDLName name cLibDLPrefix+'dlsym';
function dlclose     ( handle: pointer                          ): int32;   cdecl; external cLibDLName name cLibDLPrefix+'dlclose';

var
  uuid_generate_time_safe : function  ( out GUID: TGUID                                                                             ): int32;         cdecl;
  malloc                  : function  ( const Size: nativeuint                                                                      ): pointer;       cdecl;
  free                    : procedure ( const P: Pointer                                                                            );                cdecl;
  socket                  : function  ( const Domain: int32; const kind: int32; const protocol: int32                               ): TSocketHandle; cdecl;
  bind                    : function  ( const Socket: TSocketHandle; const Addr: pointer; const Addrlen: TSockLen                   ): int32;         cdecl;
  inet_addr               : function  ( const lpszAddress: pointer                                                                  ): TIntAddr;      cdecl;
  HtoNS                   : function  ( const Host: uint16                                                                          ): uint16;        cdecl;
  listen                  : function  ( const Socket: TSocketHandle; const BackLog: int32                                           ): int32;         cdecl;
  accept                  : function  ( const Socket: TSocketHandle; const lpAddr: pTSockAddr; var AddrLen: int32                   ): TSocketHandle; cdecl;
  connect                 : function  ( const Socket: TSocketHandle; const lpAddr: pTSockAddr; const AddrLen: int32                 ): int32;         cdecl;
  close                   : function  ( const Socket: TSocketHandle                                                                 ): int32;         cdecl;
  send                    : function  ( const Socket: TSocketHandle; const lpData: pointer; const len: int32; const flags: int32    ): int32;         cdecl;
  recv                    : function  ( const Socket: TSocketHandle; const lpData: pointer; const len: int32; const flags: int32    ): int32;         cdecl;
  shutdown                : function  ( const Socket: TSocketHandle; const Method: uint32                                           ): int32;         cdecl;
  ioctl                   : function  ( Socket: TSocketHandle; const cmd: int32; var argp: uint32                                   ): int32;         cdecl;
  clock_getres            : function  ( clk_id: clockid_t; var es: Timespec                                                         ): int32;         cdecl;
  clock_gettime           : function  ( clk_id: clockid_t; var tp: Timespec                                                         ): int32;         cdecl;
  clock_settime           : function  ( clk_id: clockid_t; const tp: Timespec                                                       ): int32;         cdecl;
  localtime               : function  ( var timep: time_t                                                                           ): ptm;           cdecl;
  toupper                 : function  ( c: uint32                                                                                   ): uint32;        cdecl;
  tolower                 : function  ( c: uint32                                                                                   ): uint32;        cdecl;

/// Pascal / Delphi like call as we can't import the symbol directly.
function GetLastError: int32;

var
  pthread_self               : function (                                                                                                     ): pthread_t; cdecl;
  pthread_mutex_init         : function ( mutex: ppthread_mutex_t; mutex_attr: ppthread_mutexattr_t                                           ): int32;     cdecl;
  pthread_mutex_destroy      : function ( mutex: ppthread_mutex_t                                                                             ): int32;     cdecl;
  pthread_mutex_lock         : function ( mutex: ppthread_mutex_t                                                                             ): int32;     cdecl;
  pthread_mutex_unlock       : function ( mutex: ppthread_mutex_t                                                                             ): int32;     cdecl;
  pthread_cond_init          : function ( cond: ppthread_cond_t; cond_atr: ppthread_condattr_t                                                ): int32;     cdecl;
  pthread_cond_wait          : function ( cond: ppthread_cond_t; mutex: ppthread_mutex_t                                                      ): int32;     cdecl;
  pthread_cond_timedwait     : function ( cond: ppthread_cond_t; mutex: ppthread_mutex_t; __abstime: ptimespec                                ): int32;     cdecl;
  pthread_cond_signal        : function ( cond: ppthread_cond_t                                                                               ): int32;     cdecl;
  pthread_cond_destroy       : function ( cond: ppthread_cond_t                                                                               ): int32;     cdecl;
  pthread_attr_init          : function ( var Attr: pthread_attr_t                                                                            ): int32;     cdecl;
  pthread_attr_setstacksize  : function ( var Attr: pthread_attr_t; StackSize: nativeuint                                                     ): int32;     cdecl;
  pthread_create             : function ( var Thread: pthread_t; var Attr: pthread_attr_t; start_routine: pthread_start_routine; Arg: Pointer ): int32;     cdecl;
  pthread_attr_destroy       : function ( var Attr: pthread_attr_t                                                                            ): int32;     cdecl;
  pthread_join               : function ( thread: pthread_t; value_ptr: pointer                                                               ): int32;     cdecl;

{$endif}
implementation
{$ifndef MSWINDOWS}
uses
  cwUnicodeCodec;

type
  ErrNo = ^int32;

var
  ext_errno: function: pointer; cdecl;

function GetLastError: int32;
begin
  Result := ErrNo(ext_errno)^;
end;

const
  cMaxBuffer = 255;

var
  NameBuffer: array[0..cMaxBuffer] of uint8;

function SetNameBuffer( const s: string ): boolean;
var
  idx: int32;
  top: nativeuint;
  CP: TUnicodeCodepoint;
  Buffer: uint32;
  pNameBuffer: ^uint8;
  Size: uint8;
  TotalBytes: nativeuint;
begin
  Result := False;
  {$ifdef NEXTGEN}
  idx := 0;
  top := pred(length(s));
  {$else}
  idx := 1;
  top := length(s);
  {$endif}
  pNameBuffer := @NameBuffer[0];
  TotalBytes := 0;
  while (idx<=top) do begin
    Result := TUnicodeCodec.DecodeCodepointFromString( CP, UnicodeString(S), idx ); //<- idx is incremented as side effect
    if not Result then exit;
    Result := TUnicodeCodec.UTF8Encode(CP,Buffer,Size);
    if not Result then exit;
    TotalBytes := TotalBytes + Size;
    if TotalBytes+Size>cMaxBuffer then begin
      Result := False;
      exit;
    end;
    Move(Buffer,pNameBuffer^,Size);
    pNameBuffer := pointer( nativeuint(pNameBuffer) + size );
  end;
  pNameBuffer^ := 0;
  Result := True;
end;

const
  {$ifdef MACOS}
    libc_Name         = '/usr/lib/libSystem.dylib' );
    libc_Prefix       =  {$ifdef CPU32BITS} '_' {$else} '' {$endif};
  {$endif}

  {$ifdef LINUX}
    libc_Name         = 'libc.so.6';
    libc_Prefix       = '';

    libUUID_Name      = 'libuuid.so.1';
    libUUID_Prefix    = '';

    libThread_Name    = 'libpthread.so.0';
    libThread_Prefix  = '';
  {$endif}

var
  libc_Handle         : pointer = nil;
  libUUID_Handle      : pointer = nil;
  libThread_Handle    : pointer = nil;

function LoadLibrary( const libName: string ): pointer;
begin
  Result := nil;
  if not SetNameBuffer( libName ) then begin
    Writeln('!!! Library name utf-conversion error: '+libName );
    exit;
  end;
  Result := dlOpen( @NameBuffer[0], cRTLD_LAZY );
  if not assigned(Result) then begin
    Writeln('!!! Could not load (file not found?): ',libName);
  end;
end;

procedure LoadLibraries;
begin
  libc_Handle      := LoadLibrary( libc_Name       );
  libUUID_Handle   := LoadLibrary( libUUID_Name    );
  libThread_Handle := LoadLibrary( libThread_Name  );
end;

procedure UnLoadLibraries;
begin
  if assigned(libc_Handle)       then dlClose(libc_Handle);
  if assigned(libUUID_Handle)    then dlClose(libUUID_Handle);
  if assigned(libThread_Handle)  then dlClose(libThread_Handle);
end;

function LoadFunction( const LibHandle: pointer; const symbolName: string ): pointer;
begin
  Result := nil;
  if not SetNameBuffer(symbolName) then begin
    Writeln('!!! Symbol name utf-conversion error: '+symbolName );
    exit;
  end;
  Result := dlSym( LibHandle, @NameBuffer[0] );
  if not assigned(Result) then begin
    Writeln( '!!! Could not locate symbol ', symbolName );
  end;
end;

procedure LoadFunctions;
const
    _errno
      {$ifdef LINUX}                =  libc_Prefix + '__errno_location'  {$endif}
      {$ifdef MACOS}                =  libc_Prefix + '__error'           {$endif}
      {$ifdef ANDROID}              =  libc_Prefix + '__errno'           {$endif};
begin
  if assigned(libc_Handle) then begin
    {$ifdef fpc } pointer(ext_errno)                 {$else} ext_errno                 {$endif} := LoadFunction( libc_Handle, _errno                                    );
    {$ifdef fpc } pointer(malloc)                    {$else} malloc                    {$endif} := LoadFunction( libc_Handle, libc_Prefix + 'malloc'                    );
    {$ifdef fpc } pointer(free)                      {$else} free                      {$endif} := LoadFunction( libc_Handle, libc_Prefix + 'free'                      );
    {$ifdef fpc } pointer(socket)                    {$else} socket                    {$endif} := LoadFunction( libc_Handle, libc_Prefix + 'socket'                    );
    {$ifdef fpc } pointer(bind)                      {$else} bind                      {$endif} := LoadFunction( libc_Handle, libc_Prefix + 'bind'                      );
    {$ifdef fpc } pointer(inet_addr)                 {$else} inet_addr                 {$endif} := LoadFunction( libc_Handle, libc_Prefix + 'inet_addr'                 );
    {$ifdef fpc } pointer(HtoNS)                     {$else} HtoNS                     {$endif} := LoadFunction( libc_Handle, libc_Prefix + 'htons'                     );
    {$ifdef fpc } pointer(listen)                    {$else} listen                    {$endif} := LoadFunction( libc_Handle, libc_Prefix + 'listen'                    );
    {$ifdef fpc } pointer(accept)                    {$else} accept                    {$endif} := LoadFunction( libc_Handle, libc_Prefix + 'accept'                    );
    {$ifdef fpc } pointer(connect)                   {$else} connect                   {$endif} := LoadFunction( libc_Handle, libc_Prefix + 'connect'                   );
    {$ifdef fpc } pointer(close)                     {$else} close                     {$endif} := LoadFunction( libc_Handle, libc_Prefix + 'close'                     );
    {$ifdef fpc } pointer(send)                      {$else} send                      {$endif} := LoadFunction( libc_Handle, libc_Prefix + 'send'                      );
    {$ifdef fpc } pointer(recv)                      {$else} recv                      {$endif} := LoadFunction( libc_Handle, libc_Prefix + 'recv'                      );
    {$ifdef fpc } pointer(shutdown)                  {$else} shutdown                  {$endif} := LoadFunction( libc_Handle, libc_Prefix + 'shutdown'                  );
    {$ifdef fpc } pointer(ioctl)                     {$else} ioctl                     {$endif} := LoadFunction( libc_Handle, libc_Prefix + 'ioctl'                     );
    {$ifdef fpc } pointer(clock_getres)              {$else} clock_getres              {$endif} := LoadFunction( libc_Handle, libc_Prefix + 'clock_getres'              );
    {$ifdef fpc } pointer(clock_gettime)             {$else} clock_gettime             {$endif} := LoadFunction( libc_Handle, libc_Prefix + 'clock_gettime'             );
    {$ifdef fpc } pointer(clock_settime)             {$else} clock_settime             {$endif} := LoadFunction( libc_Handle, libc_Prefix + 'clock_settime'             );
    {$ifdef fpc } pointer(localtime)                 {$else} localtime                 {$endif} := LoadFunction( libc_Handle, libc_Prefix + 'localtime'                 );
    {$ifdef fpc } pointer(toupper)                   {$else} toupper                   {$endif} := LoadFunction( libc_Handle, libc_Prefix + 'toupper'                   );
    {$ifdef fpc } pointer(tolower)                   {$else} tolower                   {$endif} := LoadFunction( libc_Handle, libc_Prefix + 'tolower'                   );
  end;
  if assigned(libThread_Handle) then begin
    {$ifdef fpc } pointer(pthread_self)              {$else} pthread_self              {$endif} := LoadFunction( libThread_Handle, libThread_Prefix + 'pthread_self'              );
    {$ifdef fpc } pointer(pthread_mutex_init)        {$else} pthread_mutex_init        {$endif} := LoadFunction( libThread_Handle, libThread_Prefix + 'pthread_mutex_init'        );
    {$ifdef fpc } pointer(pthread_mutex_destroy)     {$else} pthread_mutex_destroy     {$endif} := LoadFunction( libThread_Handle, libThread_Prefix + 'pthread_mutex_destroy'     );
    {$ifdef fpc } pointer(pthread_mutex_lock)        {$else} pthread_mutex_lock        {$endif} := LoadFunction( libThread_Handle, libThread_Prefix + 'pthread_mutex_lock'        );
    {$ifdef fpc } pointer(pthread_mutex_unlock)      {$else} pthread_mutex_unlock      {$endif} := LoadFunction( libThread_Handle, libThread_Prefix + 'pthread_mutex_unlock'      );
    {$ifdef fpc } pointer(pthread_cond_init)         {$else} pthread_cond_init         {$endif} := LoadFunction( libThread_Handle, libThread_Prefix + 'pthread_cond_init'         );
    {$ifdef fpc } pointer(pthread_cond_wait)         {$else} pthread_cond_wait         {$endif} := LoadFunction( libThread_Handle, libThread_Prefix + 'pthread_cond_wait'         );
    {$ifdef fpc } pointer(pthread_cond_timedwait)    {$else} pthread_cond_timedwait    {$endif} := LoadFunction( libThread_Handle, libThread_Prefix + 'pthread_cond_timedwait'    );
    {$ifdef fpc } pointer(pthread_cond_signal)       {$else} pthread_cond_signal       {$endif} := LoadFunction( libThread_Handle, libThread_Prefix + 'pthread_cond_signal'       );
    {$ifdef fpc } pointer(pthread_cond_destroy)      {$else} pthread_cond_destroy      {$endif} := LoadFunction( libThread_Handle, libThread_Prefix + 'pthread_cond_destroy'      );
    {$ifdef fpc } pointer(pthread_attr_init)         {$else} pthread_attr_init         {$endif} := LoadFunction( libThread_Handle, libThread_Prefix + 'pthread_attr_init'         );
    {$ifdef fpc } pointer(pthread_attr_destroy)      {$else} pthread_attr_destroy      {$endif} := LoadFunction( libThread_Handle, libThread_Prefix + 'pthread_attr_destroy'      );
    {$ifdef fpc } pointer(pthread_create)            {$else} pthread_create            {$endif} := LoadFunction( libThread_Handle, libThread_Prefix + 'pthread_create'            );
    {$ifdef fpc } pointer(pthread_attr_setstacksize) {$else} pthread_attr_setstacksize {$endif} := LoadFunction( libThread_Handle, libThread_Prefix + 'pthread_attr_setstacksize' );
    {$ifdef fpc } pointer(pthread_join)              {$else} pthread_join              {$endif} := LoadFunction( libThread_Handle, libThread_Prefix + 'pthread_join'              );
  end;
  if assigned(libUUID_Handle) then begin
    {$ifdef fpc } pointer(uuid_generate_time_safe)   {$else} uuid_generate_time_safe   {$endif} := LoadFunction( libUUID_Handle, libUUID_Prefix + 'uuid_generate_time_safe'  );
  end;
end;

initialization
  LoadLibraries;
  LoadFunctions;

finalization
  UnloadLibraries;

{$endif}
end.
