
///  <license>
///
///  Copyright 2021 ChapmanWorld LLC ( https://chapmanworld.com )
///
///  Redistribution and use in source and binary forms, with or without modification,
///  are permitted provided that the following conditions are met:
///
///  1. Redistributions of source code must retain the above copyright notice,
///     this list of conditions and the following disclaimer.
///
///  2. Redistributions in binary form must reproduce the above copyright notice,
///     this list of conditions and the following disclaimer in the documentation and/or
///     other materials provided with the distribution.
///
///  3. Neither the name of the copyright holder nor the names of its contributors may be
///     used to endorse or promote products derived from this software without specific prior
///     written permission.
///
///  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR
///  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
///  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
///  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
///  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
///  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
///  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
///  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
///
///  </license>
unit cwCollections.Enumerator.Standard;
{$ifdef fpc}{$mode delphiunicode}{$endif}

interface

type
  TEnumerator< T > = class( TInterfacedObject, {$ifndef fpc} IEnumerator, {$endif} IEnumerator< T > )
  type
    arrItems = array of T;
    ptrItems = ^arrItems;
  private
    fItems: ptrItems;
    fCount: nativeuint;
    fStarted: boolean;
    fItemIndex: nativeuint;
  strict private //- IEnumerator< T > -//
    {$ifdef fpc}
    function GetCurrent: T;
    {$else}
    function GenericGetCurrent: T;
    function GetCurrent: TObject;
    function IEnumerator<T>.GetCurrent = GenericGetCurrent;
    {$endif}
    function MoveNext: boolean;
    procedure Reset;
  public
    constructor Create( const Items: pointer; const Count: nativeuint );
  end;

implementation

{$ifdef fpc}
function TEnumerator<T>.GetCurrent: T;
{$else}
function TEnumerator<T>.GenericGetCurrent: T;
{$endif}
begin
  Result := fItems^[ fItemIndex ];
end;

{$ifndef fpc}
function TEnumerator<T>.GetCurrent: TObject;
begin
  Result := TEnumerator<TObject>( Self ).GenericGetCurrent;
end;
{$endif}

function TEnumerator<T>.MoveNext: boolean;
begin
  Result := False;
  if fStarted then begin
    if succ( fItemIndex ) >= fCount then exit;
    inc( fItemIndex );
  end else begin
    if fCount = 0 then exit;
    fStarted := True;
  end;
  Result := True;
end;

procedure TEnumerator<T>.Reset;
begin
  fStarted   := False;
  fItemIndex := 0;
end;

constructor TEnumerator<T>.Create(const Items: pointer; const Count: nativeuint);
begin
  inherited Create;
  fItems     := Items;
  fCount     := Count;
  Reset;
end;


end.

