
///  <license>
///
///  Copyright 2021 ChapmanWorld LLC ( https://chapmanworld.com )
///
///  Redistribution and use in source and binary forms, with or without modification,
///  are permitted provided that the following conditions are met:
///
///  1. Redistributions of source code must retain the above copyright notice,
///     this list of conditions and the following disclaimer.
///
///  2. Redistributions in binary form must reproduce the above copyright notice,
///     this list of conditions and the following disclaimer in the documentation and/or
///     other materials provided with the distribution.
///
///  3. Neither the name of the copyright holder nor the names of its contributors may be
///     used to endorse or promote products derived from this software without specific prior
///     written permission.
///
///  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR
///  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
///  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
///  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
///  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
///  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
///  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
///  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
///
///  </license>
unit cwDynlib.HandledObject.Standard;
{$ifdef fpc}{$mode delphiunicode}{$endif}

interface
uses
  cwTypes
, cwDynlib
;

type
  (* Internal type representing an actual handle, which is pointed to by the
     handle data type in deHandles *)
  THandle = record
    _signature: TGUID;
    _object: TObject;
    _extra1: pointer;
    _extra2: pointer;
  end;
  pHandle = ^THandle;

(* Constants used to represent the state of a handle *)
const
  cValidGUID: TGUID   = '{9BCFD85C-B727-4046-B251-7AE30F43ED86}';
  cInvalidGUID: TGUID = '{00000000-0000-0000-0000-000000000000}';

(* Actual handled object implementation, aliased in deHandles.Standard *)

type
  THandledObject = class( TInterfacedObject, IHandledObject )
  private
    fHandle: pHandle;
  protected //- IHandledObject -// (must be protected for descendants to see it)
    function getHandle: Handle;
  public
    constructor Create; reintroduce;
    destructor Destroy; override;
  end;

implementation

constructor THandledObject.Create;
begin
  inherited Create;
  fHandle := nil;
end;

destructor THandledObject.Destroy;
begin
  if assigned(fHandle) then begin
    fHandle^._Signature := cInvalidGUID;
    fHandle^._object := nil;
    Dispose(fHandle);
  end;
  inherited Destroy;
end;

function THandledObject.getHandle: Handle;
begin
  if not assigned(fHandle) then begin
    fHandle := new(pHandle);
    fHandle^._signature := cValidGUID;
    fHandle^._object := Self;
    fHandle^._extra1 := nil;
    fHandle^._extra2 := nil;
  end;
  Result := Handle(fHandle);
end;

end.


