﻿
///  <license>
///
///  Copyright 2021 ChapmanWorld LLC ( https://chapmanworld.com )
///
///  Redistribution and use in source and binary forms, with or without modification,
///  are permitted provided that the following conditions are met:
///
///  1. Redistributions of source code must retain the above copyright notice,
///     this list of conditions and the following disclaimer.
///
///  2. Redistributions in binary form must reproduce the above copyright notice,
///     this list of conditions and the following disclaimer in the documentation and/or
///     other materials provided with the distribution.
///
///  3. Neither the name of the copyright holder nor the names of its contributors may be
///     used to endorse or promote products derived from this software without specific prior
///     written permission.
///
///  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR
///  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
///  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
///  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
///  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
///  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
///  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
///  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
///
///  </license>
/// <summary>
///   Windows specific IHeap implementation.
/// </summary>
unit cwHeap.Heap.Windows;
{$ifdef fpc}{$mode delphiunicode}{$endif}

interface
{$ifdef MSWINDOWS}
uses
  cwHeap
;

type
  THeap = class( TInterfacedObject, IHeap )
  strict private //- IHeap -//
    function Allocate( const Size: nativeuint ): pointer;
    function Deallocate( const P: pointer ): boolean;
  end;

{$endif}
implementation
{$ifdef MSWINDOWS}
uses
  cwWin32
;

function THeap.Allocate( const Size: NativeUint  ): pointer;
begin
  Result := HeapAlloc( GetProcessHeap, HEAP_GENERATE_EXCEPTIONS, Size );
end;

function THeap.Deallocate( const P: pointer ): boolean;
begin
  Result := HeapFree(GetProcessHeap,0,p);
end;

{$endif}
end.

