﻿
///  <license>
///
///  Copyright 2021 ChapmanWorld LLC ( https://chapmanworld.com )
///
///  Redistribution and use in source and binary forms, with or without modification,
///  are permitted provided that the following conditions are met:
///
///  1. Redistributions of source code must retain the above copyright notice,
///     this list of conditions and the following disclaimer.
///
///  2. Redistributions in binary form must reproduce the above copyright notice,
///     this list of conditions and the following disclaimer in the documentation and/or
///     other materials provided with the distribution.
///
///  3. Neither the name of the copyright holder nor the names of its contributors may be
///     used to endorse or promote products derived from this software without specific prior
///     written permission.
///
///  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR
///  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
///  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
///  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
///  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
///  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
///  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
///  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
///
///  </license>
/// <summary>
///   cwTest unit testing framework from ChapmanWorld LLC.
/// </summary>
unit cwTest.RegisteredTestCase.Standard;
{$ifdef fpc}{$mode delphiunicode}{$endif}


interface
uses
  cwTest
;


type
  TRegisteredTestCase = class( TInterfacedObject, IRegisteredTestCase )
  private
    fTestCaseClass: TTestCaseClass;
    fName: string;
    fEnabled: boolean;
    fTests: array of IRegisteredTest;
  strict private //- IRegisteredTestCase -//
    function Count: nativeuint;
    function getTestByName( const name: string ): IRegisteredTest;
    function getTest( const idx: nativeuint ): IRegisteredTest;
    function CaseClass: TTestCaseClass;
    function Name: string;
    function getEnabled: boolean;
    procedure setEnabled( const value: boolean );
  public
    constructor Create( const TestCaseClass: TTestCaseClass; const TestNames: array of string );
    destructor Destroy; override;
  end;

implementation
uses
  cwTest.RegisteredTest.Standard
;

function TRegisteredTestCase.CaseClass: TTestCaseClass;
begin
  Result := fTestCaseClass;
end;

function TRegisteredTestCase.Count: nativeuint;
begin
  Result := Length(fTests);
end;

constructor TRegisteredTestCase.Create(const TestCaseClass: TTestCaseClass; const TestNames: array of string);
var
  idx: nativeuint;
begin
  inherited Create;
  fTestCaseClass := TestCaseClass;
  fEnabled := True;
  fName := string(fTestCaseClass.ClassName);
  SetLength(fTests,Length(TestNames));
  if Length(TestNames)>0 then begin
    for idx := 0 to pred(Length(TestNames)) do begin
      fTests[idx] := TRegisteredTest.Create(TestNames[idx]);
    end;
  end;
end;

destructor TRegisteredTestCase.Destroy;
begin
  SetLength(fTests,0);
  inherited Destroy;
end;

function TRegisteredTestCase.getEnabled: boolean;
begin
  Result := fEnabled;
end;

function TRegisteredTestCase.getTest( const idx: nativeuint ): IRegisteredTest;
begin
  Result := fTests[idx];
end;

function TRegisteredTestCase.getTestByName( const name: string ): IRegisteredTest;
var
  idx: nativeuint;
begin
  Result := nil;
  if Length(fTests)=0 then exit;
  for idx := 0 to pred(Length(fTests)) do begin
    if name=fTests[idx].Name then begin
      Result := fTests[idx];
      exit;
    end;
  end;
end;

function TRegisteredTestCase.Name: string;
begin
  Result := fName;
end;

procedure TRegisteredTestCase.setEnabled(const value: boolean);
begin
  fEnabled := Value;
end;

end.
