﻿
///  <license>
///
///  Copyright 2021 ChapmanWorld LLC ( https://chapmanworld.com )
///
///  Redistribution and use in source and binary forms, with or without modification,
///  are permitted provided that the following conditions are met:
///
///  1. Redistributions of source code must retain the above copyright notice,
///     this list of conditions and the following disclaimer.
///
///  2. Redistributions in binary form must reproduce the above copyright notice,
///     this list of conditions and the following disclaimer in the documentation and/or
///     other materials provided with the distribution.
///
///  3. Neither the name of the copyright holder nor the names of its contributors may be
///     used to endorse or promote products derived from this software without specific prior
///     written permission.
///
///  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR
///  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
///  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
///  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
///  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
///  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
///  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
///  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
///
///  </license>
unit cwThreading.ForTaskDelegate.Standard;
{$ifdef fpc}
  {$mode delphiunicode}
  {$MODESWITCH NESTEDPROCVARS}
{$endif}

interface
uses
  cwTypes
, cwThreading
;

type
  TForTaskDelegate = class( TInterfacedObject, ITaskDelegate )
  private
    fStart: nativeuint;
    fStop: nativeuint;
    fParameters: array of TVariant;
    fLoopDelegate: TLoopDelegate;
  strict private //- ITaskDelegate -//
    procedure Execute;
  public
    constructor Create( const Start: nativeuint; const Stop: nativeuint; const LoopDelegate: TLoopDelegate; const Parameters: array of TVariant );
    destructor Destroy; override;
  end;

implementation

constructor TForTaskDelegate.Create(const Start, Stop: nativeuint; const LoopDelegate: TLoopDelegate; const Parameters: array of TVariant);
var
  idx: nativeuint;
begin
  inherited Create;
  fStart := Start;
  fStop := Stop;
  fLoopDelegate := LoopDelegate;
  SetLength(fParameters,Length(Parameters));
  if Length(Parameters)>0 then begin
    for idx := 0 to pred(Length(Parameters)) do begin
      fParameters[idx] := Parameters[idx];
    end;
  end;
end;

destructor TForTaskDelegate.Destroy;
begin
  inherited Destroy;
  SetLength(fParameters,0);
end;

procedure TForTaskDelegate.Execute;
begin
  fLoopDelegate.Execute( fStart, fStop, fParameters );
end;

end.

