
///  <license>
///
///  Copyright 2021 ChapmanWorld LLC ( https://chapmanworld.com )
///
///  Redistribution and use in source and binary forms, with or without modification,
///  are permitted provided that the following conditions are met:
///
///  1. Redistributions of source code must retain the above copyright notice,
///     this list of conditions and the following disclaimer.
///
///  2. Redistributions in binary form must reproduce the above copyright notice,
///     this list of conditions and the following disclaimer in the documentation and/or
///     other materials provided with the distribution.
///
///  3. Neither the name of the copyright holder nor the names of its contributors may be
///     used to endorse or promote products derived from this software without specific prior
///     written permission.
///
///  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR
///  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
///  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
///  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
///  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
///  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
///  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
///  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
///
///  </license>
unit cwThreading.PromiseAction.Standard;
{$ifdef fpc}
  {$mode delphiunicode}
  {$modeswitch nestedprocvars}
{$endif}

interface
uses
  cwTypes
{$ifndef fpc}
, RTTI
{$endif}
;

type
  TPromiseActionStatus = (
    psPending,     ///    This action is pending execution.
    psFailed,      ///    The promise failed to execute.
    psCompleted    ///    The promise executed successfully.
  );

{$ifdef fpc}
type
  TPromisedMethod = procedure is nested;
{$endif}

type
  IPromiseAction = interface
    ['{3B324EEF-5DC9-44B8-A7A0-5E38EA68F7C6}']
    function getStatus: TPromiseActionStatus;
    procedure setStatus( const Value: TPromiseActionStatus );
    {$ifdef fpc}
    function getMethod: TPromisedMethod;
    procedure setMethod( const Value: TPromisedMethod );
    {$else}
    procedure setMethodName( const Value: string );
    function getMethodName: string;
    function getArgCount: nativeuint;
    function getMethodArgs: TArray<TValue>;
    procedure setMethodArgs( const Value: TArray<TValue> );
    function getMethodReturn: TValue;
    procedure setMethodReturn( const Value: TValue );
    {$endif}
  end;

type
  TPromiseAction = class( TInterfacedObject, IPromiseAction )
  private
    fStatus: TPromiseActionStatus;
  {$ifdef fpc}
    fMethod: TPromisedMethod;
  {$else}
    fMethodName: string;
    fArgs: TArray<TValue>;
    fReturn: TValue;
  {$endif}
  strict private //- IPromiseAction -//
    function getStatus: TPromiseActionStatus;
    procedure setStatus( const Value: TPromiseActionStatus );
  {$ifdef fpc}
    procedure setMethod( const Value: TPromisedMethod );
    function getMethod: TPromisedMethod;
  {$else}
    procedure setMethodName( const Value: string );
    function getMethodName: string;
    function getArgCount: nativeuint;
    function getMethodArgs: TArray<TValue>;
    procedure setMethodArgs( const Value: TArray<TValue> );
    function getMethodReturn: TValue;
    procedure setMethodReturn( const Value: TValue );
  {$endif}
    constructor Create;
  end;


implementation

function TPromiseAction.getStatus: TPromiseActionStatus;
begin
  Result := fStatus;
end;

procedure TPromiseAction.setStatus(const Value: TPromiseActionStatus);
begin
  fStatus := Value;
end;

{$ifdef fpc}

procedure TPromiseAction.setMethod(const Value: TPromisedMethod);
begin
  fMethod := Value;
end;

function TPromiseAction.getMethod: TPromisedMethod;
begin
  Result := fMethod;
end;

{$endif}

constructor TPromiseAction.Create;
begin
  inherited Create;
  fStatus := TPromiseActionStatus.psPending;
end;


{$ifndef fpc}

function TPromiseAction.getMethodName: string;
begin
  Result := fMethodName;
end;

function TPromiseAction.getArgCount: nativeuint;
begin
  Result := Length( fArgs );
end;

function TPromiseAction.getMethodArgs: TArray<TValue>;
var
  idx: nativeuint;
begin
  SetLength( Result, Length( fArgs ) );
  if Length( Result ) = 0 then exit;
  for idx := 0 to pred( Length( fArgs ) ) do Result[ idx ] := fArgs[ idx ];
end;

procedure TPromiseAction.setMethodArgs( const Value: TArray<TValue> );
var
  idx: nativeuint;
begin
  SetLength( fArgs, Length( Value ) );
  if Length( fArgs ) = 0 then exit;
  for idx := 0 to pred( Length( fArgs ) ) do fArgs[ idx ] := Value[ idx ];
end;

procedure TPromiseAction.setMethodName(const Value: string);
begin
  fMethodName := Value;
end;

function TPromiseAction.getMethodReturn: TValue;
begin
  Result := fReturn;
end;

procedure TPromiseAction.setMethodReturn( const Value: TValue );
begin
  fReturn := Value;
end;

{$endif}

end.

